const express = require("express");
const { validationResult } = require("express-validator")
const { errorHandler } = require("../src/errorHandler");
const checkFactory = require("../src/validationCheckFactory");
const { getSceneHandler } = require("../src/getSceneHandler");
const { getCapabilitiesHandler } = require("../src/getCapabilitiesHandler");
const redirectTo = require("../src/redirectTo");

let router = express.Router();

const requestChecks = checkFactory.getChecksFor("request");

router.route("/").get(requestChecks, (req, res) => {

  const errors = validationResult(req);
  if (!errors.isEmpty()) {
    let exception = errorHandler(errors);
    res.set("Content-Type", "application/xml");
    return res.status(400).send(exception);
  }

  redirectTo(req.query.request, req, res);

});

const capabilitiesChecks = checkFactory.getChecksFor("capabilities");

router.route("/capabilities").get(capabilitiesChecks, (req, res) => {

  const errors = validationResult(req);
  if (!errors.isEmpty()) {
    let exception = errorHandler(errors);
    res.set("Content-Type", "application/xml");
    return res.status(400).send(exception);
  }

  getCapabilitiesHandler(req, res);

});

const sceneChecks = checkFactory.getChecksFor("scene");

router.route("/scene").get(sceneChecks, (req, res) => {

  const errors = validationResult(req);
  if (!errors.isEmpty()) {
    let exception = errorHandler(errors);
    res.set("Content-Type", "application/xml");
    return res.status(400).send(exception);
  }

  getSceneHandler(req, res);

});


module.exports = router;